/// Data structs for rested models.
mod db;

pub struct AppState {
    pub db: sqlx::sqlite::SqlitePool,
}

impl AppState {
    pub async fn new() -> Self {
        let db = db::init("sqlite://../rested.db").await;
        Self { db }
    }
}

#[tauri::command]
async fn create_workspace(app: tauri::State<'_, AppState>, name: &str) -> Result<(), String> {
    log::info!("creating workspace");
    match sqlx::query!("INSERT INTO workspaces (name) VALUES (?)", name)
        .execute(&app.db)
        .await
    {
        Ok(_) => Ok(()),
        Err(e) => Err(e.to_string()),
    }
}

// Learn more about Tauri commands at https://tauri.app/develop/calling-rust/
#[tauri::command]
fn greet(name: &str) -> String {
    format!("Hello, {}! You've been greeted from Rust!", name)
}

#[tauri::command]
fn eval(str: &str) {
    log::info!("Evaluating: {}", str);
    // let mut ctx = boa_engine::Context::default();
    // let res = ctx
    //     .eval(boa_engine::Source::from_bytes(str.as_bytes()))
    //     .unwrap();
}

#[cfg_attr(mobile, tauri::mobile_entry_point)]
pub async fn run() {
    let state = AppState::new().await;
    tauri::Builder::default()
        .setup(|app| {
            tauri::Manager::manage(app, state);
            Ok(())
        })
        .plugin(tauri_plugin_opener::init())
        .plugin(tauri_plugin_log::Builder::new().build())
        .invoke_handler(tauri::generate_handler![greet, eval, create_workspace])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
