use leptos::task::spawn_local;
use leptos::{ev::SubmitEvent, prelude::*};
use serde::{Deserialize, Serialize};
use wasm_bindgen::prelude::*;

// Import a css file's classes:
stylance::import_crate_style!(header_styles, "src/styles/header.module.css");

#[wasm_bindgen]
extern "C" {
    #[wasm_bindgen(js_namespace = ["window", "__TAURI__", "core"])]
    async fn invoke(cmd: &str, args: JsValue) -> JsValue;
}

#[derive(Serialize, Deserialize)]
struct Eval<'a> {
    str: &'a str,
}

#[derive(Serialize, Deserialize)]
struct Name<'a> {
    name: &'a str,
}

#[component]
pub fn App() -> impl IntoView {
    let (name, set_name) = signal(String::new());
    let (greet_msg, set_greet_msg) = signal(String::new());

    let update_name = move |ev| {
        let v = event_target_value(&ev);
        set_name.set(v);
    };

    leptos::logging::log!("hello world!");

    let greet = move |ev: SubmitEvent| {
        ev.prevent_default();
        spawn_local(async move {
            let name = name.get_untracked();
            if name.is_empty() {
                return;
            }
            let args = serde_wasm_bindgen::to_value(&Name { name: &name }).unwrap();
            // Learn more about Tauri commands at https://tauri.app/develop/calling-rust/
            let new_msg = match invoke("create_workspace", args).await.as_string() {
                Some(msg) => msg,
                None => "Success".to_string(),
            };
            set_greet_msg.set(new_msg);
        });
    };

    view! {
        <Header />
        <main class="container">
            <form class="row" on:submit=greet>
                <input
                    id="greet-input"
                    placeholder="Enter a name..."
                    on:input=update_name
                />
                <button type="submit">"Greet"</button>
            </form>
            <p>{ move || greet_msg.get() }</p>
        </main>
    }
}

#[component]
pub fn Header() -> impl IntoView {
    view! {
        <header class=header_styles::header>
            <h1 class=header_styles::title>"rested"</h1>

            <nav class=header_styles::nav>
                <a href="#">"Home"</a>
                <a href="#">"Settings"</a>
            </nav>
        </header>
    }
}
